@@ -1,5 +1,5 @@
-#include <RSQLite.h>
-#include "sqlite3.h"
+#include "pch.h"
+#include "sqlite3-cpp.h"
 
 //' RSQLite version
 //'
